/***************************************************************
*
*  Module:       cdr-test.c
*
*  Description:  Sample code that shows how to interface to
*                the cd-4810.sys driver. This driver supports
*                CashDrawer directly connected to IBM
*                SurePOS 300 4810-32x\33x\34x models.
*
*  DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
*  sample code created by IBM Corporation. This sample code is not
*  part of any standard or IBM product and is provided to you solely
*  for  the purpose of assisting you in the development of your
*  applications.  The code is provided "AS IS", without
*  warranty of any kind.  IBM shall not be liable for any damages
*  arising out of your use of the sample code, even if they have been
*  advised of the possibility of such damages.
*
****************************************************************/

#include <stdio.h>
//#include <memory.h>
//#include <dos.h>
//#include <stdlib.h>
//#include <io.h>
#include <fcntl.h>

#define CD1 1
#define CD2 2
#define CDSTATUSLEN 14

/****************************************************************
*
* Function Definitions prototype
*
****************************************************************/

int initCashDrawer(void);
int destroyCashDrawer(void);
int getCashdrawerStatus(void);
int OpenCashdrawer(int nCD);
int encodeStatusByte(char rawStatusByte[], int dDataLength);
int dSleepDuration = 5; //in seconds
void sleep(unsigned int);
int pCD;

/****************************************************************
*
* Function:    Main
* Description: Entry point for code to start
* Argument:    None
* Returns:     NULL
*
****************************************************************/

void main()
{
   int dOption = 0;

   printf("Welcome!\n");

   while (dOption != 9)
   {
       printf("Please enter\n");
       printf("1 to check CashDrawer status\n");
       printf("2 to open CashDrawer A \n");
       printf("3 to open CashDrawer B \n");
       printf("9 to exit\n");
       scanf("%d", &dOption);
       switch (dOption)
       {
           case 1: getCashdrawerStatus();
                   break;

           case 2: OpenCashdrawer(CD1);
                   break;

           case 3: OpenCashdrawer(CD2);
                   break;

           default:
                   printf("Thanks.");
                   break;
       }

   }

   return;
}

void sleep(unsigned int seconds)
{
  long tnow, tthen;
  time(&tnow);
  tthen = tnow + seconds;
  while (1)
  {
    time(&tnow);
    if (tnow >= tthen)
      return;
  }
}

/***************************************************************
*
*  Function:    initCashDrawer
*  Description: Initialize CashDrawer using fopen method
*               It assume logical name to be CDWR
*  Argument:    None
*  Returns:     0 for success and 1 for Failure
*
****************************************************************/

int initCashDrawer()
{
    if((pCD=open("CDWR", O_RDWR|O_BINARY)) == -1)
    {
        printf("Cannot open CashDrawer! Please check the config.sys\n");
          return -1;
    }

    return 0;
}

/***************************************************************
*
*  Function:    destroyCashDrawer
*  Description: File Close the CashDrawer handle
*  Argument:    None
*  Returns:     0 for success
*
****************************************************************/


int destroyCashDrawer()
{
    close(pCD);
    return 0;
}

/***************************************************************
*
*  Function:    getCashDrawerStatus
*  Description: Read CashDrawerStatus and return to the caller
*  Argument:    None
*  Returns:     0 for Success -1 for Failure
*
****************************************************************/

int getCashdrawerStatus()
{
    char *SysCmd = "\x1B^1s";
    char statusByte[40];
    int i;
    int dRetcode = -1;
    int rc = -1;

    memset(statusByte,0x0,40);

    /* issue system command */
    if (initCashDrawer()==0)
    {
        printf("Issue system command to check CashDrawer Status..\n");
        printf("Command sent =  %s\n",SysCmd);
        write(pCD,SysCmd,4);
        destroyCashDrawer();
    }

    else
    {
        destroyCashDrawer();
        return dRetcode;
    }

    /* read status byte */
    if (initCashDrawer()==0)
    {
        sleep(1);
        rc = read(pCD,statusByte,CDSTATUSLEN);
        if (rc != -1)
        {
            for (i=0;i<CDSTATUSLEN;i++)
            {
                printf("statusbyte[%d]: %x\n",i, statusByte[i]);
            }
        }
        else
        {
            printf("No data received!\n");
        }

        destroyCashDrawer();
        encodeStatusByte(statusByte, CDSTATUSLEN);
    }
    else
    {
        destroyCashDrawer();
    }

    return dRetcode;
}

/***************************************************************
*
*  Function:    OpenCashdrawer
*  Description: Open CashDrawer based on the input CashDrawer
*               number
*  Argument:    nCD - CashDrawer number
*  Returns:     0 for Success -1 for Failure
*
****************************************************************/

int OpenCashdrawer(int nCD)
{
    char *OpenCD1 ="\x1B^1O\0";
    char *OpenCD2 ="\x1B^2O\0";
    int dRetcode = -1;

    if (nCD != CD1 && nCD !=CD2)
    {
        return dRetcode;
    }

    if (initCashDrawer()==0)
    {
       if (nCD == CD1)
       {
           write(pCD,OpenCD1,4);
       }
       else
       {
          write(pCD,OpenCD2,4);
       }
       dRetcode = 0;
    }

    destroyCashDrawer();
    return dRetcode;
}
/***************************************************************
*
*  Function:    encodeStatusByte
*  Description: Get rawStatusByte from port and interpret it
*  Argument:    rawStatusByte - StatusByte obtained from port
*               dDataLength - Length of StatusByte from fread
*  Returns:     0 for Success -1 for Failure
*
****************************************************************/

int encodeStatusByte(char rawStatusByte[], int dDataLength)
{
    int dStatusByte1 ;
    int dStatusByte2 ;
    char statusByte1[5];
    char statusByte2[5];

    /* The rawStatusByte obtained must confined to a certain format */
    if (dDataLength < 14 || rawStatusByte[0] != 0x1b || rawStatusByte[1]!= 0x5e || rawStatusByte[13]!=0x52)
    {
        printf("Unexpected Status format read from device\n");
        return -1;
    }

    memcpy(statusByte1, &rawStatusByte[2], 5);
    memcpy(statusByte2, &rawStatusByte[8], 5);

    dStatusByte1 = atoi(statusByte1);
    dStatusByte2 = atoi(statusByte2);

    printf("CashDrawer Status byte 1 = %d\n", dStatusByte1);
    printf("CashDrawer Status byte 2 = %d\n", dStatusByte2);

    printf("______________________________________\n");

    if ((dStatusByte1 & 0x10) == 0x10)
    {
        printf(" CashDrawer A  -- Disconnected \n");
    }
    else
    {
        printf(" CashDrawer A  -- Connected \n");

        if ((dStatusByte1 & 0x40) == 0x40)
        {
            printf(" CashDrawer A  -- Open \n");
        }
        else
        {
            printf(" CashDrawer A  -- Close \n");
        }
    }

    if ((dStatusByte1 & 0x08) == 0x08)
    {
        printf(" CashDrawer B  -- Disconnected \n");
    }
    else
    {
        printf(" CashDrawer B  -- Connected \n");
        if ((dStatusByte1 & 0x20) == 0x20)
        {
            printf(" CashDrawer B  -- Open \n");
        }
        else
        {
            printf(" CashDrawer B  -- Close \n");
        }
    }

    printf("______________________________________\n");
    return 0;
}

